package com.faraphel.tasks_valider.ui.screen.authentication

import android.app.Activity
import android.widget.Toast
import androidx.compose.foundation.layout.*
import androidx.compose.material3.Button
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.connectivity.task.session.TaskSession
import com.faraphel.tasks_valider.database.entities.error.HttpException
import com.faraphel.tasks_valider.ui.screen.scan.qr.ScanBarcodeScreen
import com.journeyapps.barcodescanner.BarcodeResult
import okhttp3.HttpUrl.Companion.toHttpUrl


/**
 * Authentification screen where the client can give his information
 */
@Composable
fun AuthenticationClientScreen(activity: Activity, client: TaskClient, session: MutableState<TaskSession?>) {
    val controller = rememberNavController()
    val barcode = remember { mutableStateOf<BarcodeResult?>(null) }

    NavHost(navController = controller, startDestination = "main") {
        composable("main") {
            AuthenticationClientContent(activity, controller, client, session, barcode)
        }
        composable("scan") {
            if (barcode.value == null) ScanBarcodeScreen(activity, barcode)
            else controller.navigate("main")
        }
    }
}


@Composable
fun AuthenticationClientContent(
    activity: Activity,
    controller: NavController,
    client: TaskClient,
    session: MutableState<TaskSession?>,
    barcode: MutableState<BarcodeResult?>,
) {
    val cardId = remember { mutableStateOf("") }
    val password = remember { mutableStateOf("") }

    // check if the barcode contain information about the card

    var defaultCardId = ""
    if (barcode.value != null) {
        val studentUrl = barcode.value!!.text.toHttpUrl()
        cardId.value = studentUrl.pathSegments[0]
    }

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        // title
        Text(
            text = "Authentication",
            fontSize = 32.sp
        )

        // separator
        Spacer(modifier = Modifier.height(24.dp))

        // card identifier
        Row {
            // text
            TextField(
                value = cardId.value,
                placeholder = { Text("Card Id") },
                onValueChange = { text -> cardId.value = text },
            )
            // button to scan the card
            Button(onClick = {
                barcode.value = null
                controller.navigate("scan")
            }) {
                Text("Scan")
            }
        }

        // password
        TextField(
            value = password.value,
            visualTransformation = PasswordVisualTransformation(),
            placeholder = { Text("Password") },
            onValueChange = { text -> password.value = text },
        )

        // separator
        Spacer(modifier = Modifier.height(24.dp))

        // submit button

        Button(onClick = {
            Thread { authenticate(activity, client, cardId.value, password.value, session) }.start()
        }) {
            Text("Submit")
        }
    }
}

fun authenticate(
    activity: Activity,
    client: TaskClient,
    cardId: String,
    password: String,
    session: MutableState<TaskSession?>
) {
    try {
        // try to get a session from the identifiers
        session.value = client.session.newFromCardId(cardId, password)
    } catch (exception: HttpException) {
        // in case of error, show a message to the user
        when (exception.getCode()) {
            401 -> {
                activity.runOnUiThread {
                    Toast.makeText(activity, "Invalid card id or password", Toast.LENGTH_SHORT).show()
                }
            }
            else -> {
                activity.runOnUiThread {
                    Toast.makeText(activity, "Unknown error: $exception", Toast.LENGTH_SHORT).show()
                }
            }
        }
    }
}
