package com.faraphel.tasks_valider.ui.screen.task

import android.app.Activity
import android.widget.Toast
import androidx.compose.foundation.layout.*
import androidx.compose.material3.Checkbox
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.connectivity.task.session.TaskPermission
import com.faraphel.tasks_valider.database.entities.*
import com.faraphel.tasks_valider.utils.dateTimeFormatter
import com.faraphel.tasks_valider.utils.parser
import com.google.gson.reflect.TypeToken


/**
 * This screen represent a student
 * @param student the student object
 */
@Composable
fun TaskStudentScreen(
    activity: Activity,
    client: TaskClient,
    user: PersonEntity,
    student: PersonEntity,
) {
    val tasks = remember { mutableStateOf<List<TaskEntity>?>(null) }
    val validations = remember { mutableStateOf<List<ValidationEntity>?>(null) }

    if (tasks.value == null || validations.value == null)
        Thread {
            refreshTasksValidations(
                activity,
                client,
                student,
                tasks,
                validations
            )
        }.start()

    Column(
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        // title
        Text(text = "Student", fontSize = 32.sp)
        // student name - subtitle
        Text(text = student.fullName(), fontSize = 24.sp)

        // separator
        Spacer(modifier = Modifier.height(24.dp))

        // if both the list of tasks and validations are loaded
        if (!(tasks.value == null || validations.value == null)) {
            tasks.value?.forEach { task ->
                // get the validation
                val validation = validations.value!!.firstOrNull { validation -> validation.taskId == task.id }

                Box(
                    modifier = Modifier.fillMaxWidth().padding(vertical = 8.dp, horizontal = 64.dp),
                ) {
                    Row {
                        Column {
                            // task title
                            Text(text = task.title, fontWeight = FontWeight.Bold)
                            // task description
                            task.description?.let { description -> Text(description) }
                            // if the task have been validated, show the date
                            if (validation != null) Text(text = dateTimeFormatter.format(validation.date))
                        }

                        // separator
                        Spacer(modifier = Modifier.fillMaxWidth())

                        // the validation state
                        Checkbox(
                            checked = validation != null,
                            enabled = user.role.permissions.contains(TaskPermission.WRITE),
                            onCheckedChange = { state ->
                                Thread {
                                    // TODO(Faraphel): simplify or put the UI refresh in the update function ?

                                    // send a notification to the server about the validation
                                    updateValidation(
                                        client,
                                        state,
                                        validation ?: ValidationEntity(
                                            teacherId=user.id,
                                            studentId=student.id,
                                            taskId=task.id,
                                        )
                                    )
                                    // refresh the UI
                                    refreshTasksValidations(
                                        activity,
                                        client,
                                        student,
                                        tasks,
                                        validations
                                    )
                                }.start()
                            }
                        )
                    }
                }
            }
        }
    }
}

fun refreshTasksValidations(
    activity: Activity,
    client: TaskClient,
    student: PersonEntity,
    tasks: MutableState<List<TaskEntity>?>,
    validations: MutableState<List<ValidationEntity>?>,
) {
    // try to obtain the list of subject
    val allRelationsPersonSessionSubject = client.entities.relationPersonSessionSubject.getAll()
    // get the subject that the student is using
    val relationPersonSessionSubject = allRelationsPersonSessionSubject.firstOrNull { relation ->
        relation.studentId == student.id
    }

    if (relationPersonSessionSubject == null)
        // TODO(Faraphel): should be able to assign a subject ?
        return activity.runOnUiThread { Toast.makeText(activity, "No subject assigned", Toast.LENGTH_LONG).show() }

    // try to obtain the list of tasks
    val allTasks = client.entities.task.getAll()

    // get the tasks that are linked to this subject
    tasks.value = allTasks
        .filter { task -> task.subjectId == relationPersonSessionSubject.subjectId }
        .sortedBy { task -> task.order }

    // try to obtain the list of validations
    val allValidations = client.entities.validation.getAll()
    // filter only the interesting validations
    validations.value = allValidations.filter { validation ->
        validation.studentId == student.id &&
        validation.taskId in allTasks.map { task -> task.id }
    }
}


fun updateValidation(client: TaskClient, checked: Boolean, validation: ValidationEntity) {
    if (checked)
        // if the validation is not set, create it
        client.entities.validation.save(validation)
    else
        // if the validation is set, delete it
        client.entities.validation.delete(validation)
}
